rm(list=ls())

source("SRSPD20170504.R")
library(randtoolbox)
library(lhs)
library(GenSA)
library(DiceKriging)
library(DiceOptim)
library(DiceDesign)
library(RobustGaSP)

themethod = "SMEDCV"

p=2
n1=13
n2=26
seed=1
ntrial=1000

f=function(x)
{
  f1=3/4*exp(-.25*(9*x[1]-2)^2-.25*(9*x[2]-2)^2)
  f2=3/4*exp(-1/49*(9*x[1]+1)^2-1/10*(9*x[2]+1)^2)
  f3=1/2*exp(-.25*(9*x[1]-7)^2-.25*(9*x[2]-3)^2)
  f4=1/5*exp(-(9*x[1]-4)^2-(9*x[2]-7)^2)
  val=f1+f2+f3-f4
  return(val)
}

	
PEns = function(design) {
	Y = rep(0,dim(design)[1])
	for(i in 1:dim(design)[1]) Y[i] = f(design[i,1:2])
	A = rgasp(design=design, response=Y)
	set.seed(1)
	testsample = cbind(runif(10000,0,1),runif(10000,0,1))
	realY = rep(0,10000)
	for(i in 1:10000) realY[i] = f(testsample[i,1:2])
	pre = predict(object=A,testing_input=testsample)$mean
	
	mean((pre-realY)^2)
}

sumDist = matrix(0,40,7)
sumPEns = matrix(0,40,7)
sumDist2 = matrix(0,40,7)
sumPEns2 = matrix(0,40,7)

# CVa, CVc, CVp, MMLH

seed=1
for(seed in 1:ntrial){


resDist = matrix(0,40,7)
resPEns = matrix(0,40,7)


## SMED-CV

set.seed(seed)
Dii = lhsDesign(n1,p)$design
Di = maximinSA_LHS(Dii)$design
D = Di
resp = rep(0,n1)
for(i in 1:n1) resp[i] = f(D[i,1:2])

supp = cbind(runif(5000,0,1),runif(5000,0,1))


resDist[n1,1] = min(dist(D))
resPEns[n1,1] = PEns(D)


for(n2 in 1:26){  
	n = dim(D)[1]

	preds = matrix(0,5000,n+1)
	predsX = rep(0,n)
	gp = rgasp(design=D, response=resp)
	preds[,n+1]=predict(gp,supp)$mean
	for(j in 1:n){ 
		gp = rgasp(design=D[-j,], response=resp[-j])
		preds[,j]=predict(gp,supp)$mean
		predsX[j]=predict(gp,D[j,,drop=FALSE])$mean
	}
	scores = rep(0,5000)
	for(j in 1:n) scores = scores + (preds[,j]-preds[,n+1])^2
	scores = 1/(scores/n)
	scoresX = 1/((resp-predsX)^2*(n-1)/n/n)
	
	for(i in 1:5000) {
		rx = 0
		for(j in 1:n) rx = rx + scoresX[j] / (sum((supp[i,]-D[j,])^2)+10^-20)^(2*p)
		scores[i] = scores[i] * rx
	}

	addindex = order(scores)[1]
	D = rbind(D,supp[addindex,])
	resp = c(resp,f(supp[addindex,]))

	resDist[n1+n2,1] = min(dist(D))
	resPEns[n1+n2,1] = PEns(D)

	par(mfrow=c(1,1))
	plot(D,xlim=c(0,1),ylim=c(0,1))
	points(supp[order(scores)[1],2]~supp[order(scores)[1],1],pch=3,col='red')
	points(supp[order(scores)[2:100],],cex=.5,pch=3,col='yellow')
}


## 

resPEns[1,4]=1
sumDist=sumDist+resDist
sumPEns=sumPEns+resPEns
sumDist2=sumDist2+resDist^2
sumPEns2=sumPEns2+resPEns^2
print('')
print('')
print(seed)
print(sumPEns/seed)

write.table(x=sumDist,file=paste('simu3CSMEDCVrankeressumDist',p,'.txt',sep=''),quote=FALSE,row.names=FALSE,col.names=FALSE)
write.table(x=sumPEns,file=paste('simu3CSMEDCVFrankeressumPEns',p,'.txt',sep=''),quote=FALSE,row.names=FALSE,col.names=FALSE)
write.table(x=sumDist,file=paste('simu3CSMEDCVFrankeressumDist2',p,'.txt',sep=''),quote=FALSE,row.names=FALSE,col.names=FALSE)
write.table(x=sumPEns,file=paste('simu3CSMEDCVFrankeressumPEns2',p,'.txt',sep=''),quote=FALSE,row.names=FALSE,col.names=FALSE)

}

resDist=sumDist/ntrial
resPEns=sumPEns/ntrial

write.table(x=resDist,file=paste('simu3CSMEDCVFrankeresDist',p,'.txt',sep=''),quote=FALSE,row.names=FALSE,col.names=FALSE)
write.table(x=resPEns,file=paste('simu3CSMEDCVFrankeresPEns',p,'.txt',sep=''),quote=FALSE,row.names=FALSE,col.names=FALSE)

