rm(list=ls())

library(randtoolbox)
library(lhs)
library(GenSA)
library(DiceKriging)
library(DiceOptim)
library(DiceDesign)

themethod = 'MMLH-EI'
findex = 3
#for(findex in 1:4) {
ntrial=1000
maxitvalue = 100

if(findex == 0) { #Franke's function
	p=2
	n2=20
	n0=13
	N=30
	f=function(x)
	{
	  f1=3/4*exp(-.25*(9*x[1]-2)^2-.25*(9*x[2]-2)^2)
	  f2=3/4*exp(-1/49*(9*x[1]+1)^2-1/10*(9*x[2]+1)^2)
	  f3=1/2*exp(-.25*(9*x[1]-7)^2-.25*(9*x[2]-3)^2)
	  f4=1/5*exp(-(9*x[1]-4)^2-(9*x[2]-7)^2)
	  val=f1+f2+f3-f4
	  return(val)
	}
}

if(findex == 1) { # Branin function
	p=2
	n2choices=seq(3,15,3)
	n0choices=ceiling(n2choices*2/3)
	N=20
	f=function(x)
	{
		x1bar <- 15*x[1] - 5
		x2bar <- 15*x[2]
		term1 <- x2bar - 5.1*x1bar^2/(4*pi^2) + 5*x1bar/pi - 6
		term2 <- (10 - 10/(8*pi)) * cos(x1bar)
		y <- term1^2 + term2 + 10
		return(-y)
	}
}

if(findex == 2) { # GoldsteinPrice function
	p=2
	n2choices=seq(6,21,3)
	n0choices=ceiling(n2choices*2/3)
	N=40
	f=function(x)
	{
		x1bar <- 4*x[1] - 2
		x2bar <- 4*x[2] - 2
		fact1a <- (x1bar + x2bar + 1)^2
		fact1b <- 19 - 14*x1bar + 3*x1bar^2 - 14*x2bar + 6*x1bar*x2bar + 3*x2bar^2
		fact1 <- 1 + fact1a*fact1b
		fact2a <- (2*x1bar - 3*x2bar)^2
		fact2b <- 18 - 32*x1bar + 12*x1bar^2 + 48*x2bar - 36*x1bar*x2bar + 27*x2bar^2
		fact2 <- 30 + fact2a*fact2b
		prod <- fact1*fact2
		y <- (log(prod)) 
		return(-y)
	}
}

if(findex == 3) { # Hartmann3 function
	p=3
	n2choices=seq(4,16,4)
	n0choices=ceiling(n2choices*2/3)
	N=20
	f=function(x)
	{
		alpha <- c(1.0, 1.2, 3.0, 3.2)
		A <- c(3.0, 10, 30,
			 0.1, 10, 35,
			 3.0, 10, 30,
			 0.1, 10, 35)
		A <- matrix(A, 4, 3, byrow=TRUE)
		P <- 10^(-4) * c(3689, 1170, 2673,
					   4699, 4387, 7470,
					   1091, 8732, 5547,
					   381, 5743, 8828)
		P <- matrix(P, 4, 3, byrow=TRUE)
		xxmat <- matrix(rep(x,times=4), 4, 3, byrow=TRUE)
		inner <- rowSums(A[,1:3]*(xxmat-P[,1:3])^2)
		outer <- sum(alpha * exp(-inner))
		y <- -outer
		return(-y)
	}
}

if(findex == 4) { # Hartmann6 function
	p=6
	n2choices=seq(6,24,6)
	n0choices=ceiling(n2choices/2)
	N=40
	f=function(x)
	{
		alpha <- c(1.0, 1.2, 3.0, 3.2)
		A <- c(10, 3, 17, 3.5, 1.7, 8,
			 0.05, 10, 17, 0.1, 8, 14,
			 3, 3.5, 1.7, 10, 17, 8,
			 17, 8, 0.05, 10, 0.1, 14)
		A <- matrix(A, 4, 6, byrow=TRUE)
		P <- 10^(-4) * c(1312, 1696, 5569, 124, 8283, 5886,
					   2329, 4135, 8307, 3736, 1004, 9991,
					   2348, 1451, 3522, 2883, 3047, 6650,
					   4047, 8828, 8732, 5743, 1091, 381)
		P <- matrix(P, 4, 6, byrow=TRUE)
		xxmat <- matrix(rep(x,times=4), 4, 6, byrow=TRUE)
		inner <- rowSums(A[,1:6]*(xxmat-P[,1:6])^2)
		outer <- sum(alpha * exp(-inner))
		y <- - log(outer)
		return(-y)
	}
}


nf=function(x)
-f(x)

ML=function(para)
{
  n=dim(D)[1]
  k=1 
  R=exp(-para[k]*(as.matrix(dist(D[,k], diag=T, upper=T)))^2);
  for(k in 2:p) R=R *exp(-para[k]*(as.matrix(dist(D[,k], diag=T, upper=T)))^2);
  R=R+10^(-6)*I  
  eig=eigen(R)
  a=c(t(eig$vec)%*%one)
  b=c(t(eig$vec)%*%y)
  mu=sum(a*b/eig$val)/(sum(a^2/eig$val))
  sigma2=1/n*(sum(b^2/eig$val)-mu^2*sum(a^2/eig$val))
  val=n*log(sigma2)+sum(log(eig$val))+1/n^(1/p)*sum((log(para)-1/(2*p)*log(n))^2)
  return(val)
}

ok=function(x)
{
w=apply(D-one%*%t(x),1,corr)
val1=mu+sum(w*coef)
r.w=c(t(eig$vec)%*%w)
t1=sum(r.w^2/eig$val)
t2=sum(a^2/eig$val)
t3=sum(a*r.w/eig$val)
val2=sig2*(max((1-t1+(1-t3)^2/t2),0)+10^(-6))
return(c(val1,val2))
}

corr=function(e)
return(exp(-sum(theta*e^2)))

fhat=function(x)
{
w=apply(D-one%*%t(x),1,corr)
val=mu+sum(w*coef)
return(val)
}
nfhat=function(x)
-fhat(x)

ghat=function(x)
{
fv=fhat(x)
if(fv<fmin) 
{fmin=fv-10^(-6)
xmin=x}
val=fv-fmin
return(val)
}

energyhat=function(x)
{
d2=apply(D.e-one.e%*%t(x),1,eucl2)+10^(-10)
val=-2*log(ghat(x))+log(sum(exp(-2*log(ghatev.e)-4*log(d2))))
return(val)
}

EI=function(x)
{
v=ok(x)
nyhat=v[1]
s=sqrt(v[2])
u=(nfmin-nyhat)/s
val=s*(u*pnorm(u)+dnorm(u))
return(-val)
}

gopt=optim(rep(.2,p),nf)$par

eucl2=function(u)
sum(u^2)



## MMLH - EI

resmeanf = matrix(0,length(n2choices),N)
n2=n2choices[1]

for(indexn in 1:length(n2choices)) {

resdist = matrix(0,ntrial,N)
resfval = matrix(0,ntrial,N)
resf = matrix(0,ntrial,N)
reseimp = matrix(0,ntrial,N)

n2=n2choices[indexn]
n0=n0choices[indexn]

trial=1
for(trial in 1:ntrial)
{
set.seed(trial)

if(p==2) N.plot=200
if(p==2) p1=p2=seq(0,1,length=N.plot)
if(p==2) x.grid=expand.grid(p1,p2)
if(p==2) f12=apply(x.grid,1,f)
if(p==2) image(p1,p2,matrix(f12,N.plot,N.plot),col=cm.colors(5), xlab=expression(x[1]), ylab=expression(x[2]), main=paste(themethod,trial,sep=', '))
if(p==2) contour(p1,p2,matrix(f12,N.plot,N.plot),add=T, nlevels=10, lty=3)

	D0=D=maximinSA_LHS(lhsDesign(n2,p)$design)$design
if(p==2)	points(D, pch=3,col=2)

	n=dim(D)[1]
	I=diag(n)
	one=rep(1,n)
	y=apply(D,1,nf)

	me=n2^(1/(2*p))
	ini=halton(N,p)
	lo=rep(.5/N,p)
	up=rep(1-.5/N,p)
	gamma=1/(2*p)
	theta=GenSA(par=rep(me,p),fn=ML,lower=rep(me/10,p), upper=rep(10*me,p), control=list(maxit=maxitvalue))$par
	para=theta
	k=1 
	R=exp(-para[k]*(as.matrix(dist(D[,k], diag=T, upper=T)))^2);
	for(k in 2:p) R=R *exp(-para[k]*(as.matrix(dist(D[,k], diag=T, upper=T)))^2);
	R=R+10^(-6)*I  
	eig=eigen(R)
	a=c(t(eig$vec)%*%one)
	b=c(t(eig$vec)%*%y)
	mu=sum(a*b/eig$val)/(sum(a^2/eig$val))
	sig2=1/n*(sum(b^2/eig$val)-mu^2*sum(a^2/eig$val))
	coef=eig$vec%*%((b-mu*a)/eig$val)
	smedopt=D[which.min(y),]
#	smedopt=optim(c(.2,.2),nfhat,lower=c(0,0),upper=c(1,1),method="L-BFGS-B")$par
	sqrt(eucl2(smedopt-gopt))

	resdist[trial,n]=sqrt(eucl2(smedopt-gopt))
	resfval[trial,n]=f(smedopt)-f(gopt)
	resf[trial,n]=f(smedopt)

	#EI Algorithm

#	image(p1,p2,matrix(f12,N.plot,N.plot),col=cm.colors(5), xlab=expression(x[1]), ylab=expression(x[2]))
#	contour(p1,p2,matrix(f12,N.plot,N.plot),add=T, nlevels=10, lty=3)
#	points(D, pch=3,col=2)

	nfmin=min(y)

	ini=halton(N,p)
	for(j in (n2+1):N)
	{
		new=GenSA(par=ini[j,],lower=lo, upper=up, fn=EI, control=list(maxit=maxitvalue))$par
		reseimp[trial,j]=EI(new)
if(p==2) points(new[1],new[2],type="n")
if(p==2) text(new[1],new[2],labels=j, col='green', cex=.75)
		D=rbind(D,new)
		n=dim(D)[1]
		one=rep(1,n)
		I=diag(n)
		y=c(y,nf(new))
		theta=optim(theta,ML,lower=theta/2, upper=2*theta, method="L-BFGS-B")$par
		para=theta
		k=1 
		R=exp(-para[k]*(as.matrix(dist(D[,k], diag=T, upper=T)))^2);
		for(k in 2:p) R=R *exp(-para[k]*(as.matrix(dist(D[,k], diag=T, upper=T)))^2);
		R=R+10^(-6)*I  
		eig=eigen(R)
		a=c(t(eig$vec)%*%one)
		b=c(t(eig$vec)%*%y)
		mu=sum(a*b/eig$val)/(sum(a^2/eig$val))
		sig2=1/n*(sum(b^2/eig$val)-mu^2*sum(a^2/eig$val))
		coef=eig$vec%*%((b-mu*a)/eig$val)
		Rinv=eig$vec%*%diag(1/eig$val)%*%t(eig$vec)
		nfmin=min(y)
		eiopt=D[which.min(y),]
		resdist[trial,j]=sqrt(eucl2(eiopt-gopt))
		resfval[trial,j]=f(eiopt)-f(gopt)
		resf[trial,j]=f(eiopt)
	}
#	print(c(findex,trial,resf[trial,n2:N]))
#	plot(resfval[trial,n2:N]~I(n2:N))

#write.table(x=resdist,file=paste('simuTechRRdist-f',findex,themethod,'.txt',sep=''),quote=FALSE,row.names=FALSE,col.names=FALSE)
#write.table(x=resfval,file=paste('simuTechRRfval-f',findex,themethod,'.txt',sep=''),quote=FALSE,row.names=FALSE,col.names=FALSE)
#write.table(x=resf,file=paste('simuTechRR-f-f',findex,themethod,'.txt',sep=''),quote=FALSE,row.names=FALSE,col.names=FALSE)
#write.table(x=reseimp,file=paste('simuTechRReimp-f',findex,themethod,'.txt',sep=''),quote=FALSE,row.names=FALSE,col.names=FALSE)

}

for(j in 1:N) resmeanf[indexn,j] = mean(resf[,j])
print(n2)
print(resmeanf)

write.table(x=resmeanf,file=paste('simuTechRR-MN3-f',findex,themethod,'.txt',sep=''),quote=FALSE,row.names=FALSE,col.names=FALSE)

}

